@@ -1,4 +1,9 @@
-use std::{collections::HashMap, fmt, path::PathBuf};
+use std::{
+    collections::HashMap,
+    fmt::{self, Display},
+    path::PathBuf,
+    str::FromStr,
+};
 
 use tokio::fs;
 
@@ -22,11 +27,9 @@ impl GpuController {
         let mut hw_monitors = Vec::new();
 
         if let Ok(hw_mons_iter) = std::fs::read_dir(sysfs_path.join("hwmon")) {
-            for hw_mon_dir in hw_mons_iter {
-                if let Ok(hw_mon_dir) = hw_mon_dir {
-                    if let Ok(hw_mon) = HwMon::new_from_path(hw_mon_dir.path()).await {
-                        hw_monitors.push(hw_mon);
-                    }
+            for hw_mon_dir in hw_mons_iter.flatten() {
+                if let Ok(hw_mon) = HwMon::new_from_path(hw_mon_dir.path()).await {
+                    hw_monitors.push(hw_mon);
                 }
             }
         }
@@ -162,17 +165,17 @@ impl GpuController {
             let mut active = 0;
 
             for mut line in content.trim().split('\n') {
-                if let Some(stripped) = line.strip_suffix("*") {
+                if let Some(stripped) = line.strip_suffix('*') {
                     line = stripped;
 
-                    if let Some(identifier) = stripped.split(":").next() {
+                    if let Some(identifier) = stripped.split(':').next() {
                         active = identifier
                             .trim()
                             .parse()
                             .expect("Unexpected power level identifier");
                     }
                 }
-                if let Some(s) = line.split(":").last() {
+                if let Some(s) = line.split(':').last() {
                     power_levels.push(s.trim().to_string());
                 }
             }
@@ -250,9 +253,11 @@ impl Default for PerformanceLevel {
     }
 }
 
-impl PerformanceLevel {
-    pub fn from_str(power_level: &str) -> Result<Self, GpuControllerError> {
-        match power_level {
+impl FromStr for PerformanceLevel {
+    type Err = GpuControllerError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        match s {
             "auto" | "Automatic" => Ok(PerformanceLevel::Auto),
             "high" | "Highest Clocks" => Ok(PerformanceLevel::High),
             "low" | "Lowest Clocks" => Ok(PerformanceLevel::Low),
@@ -292,3 +297,14 @@ impl From<std::io::Error> for GpuControllerError {
         Self::IoError(e)
     }
 }
+
+impl Display for GpuControllerError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str(&match self {
+            GpuControllerError::NotAllowed(info) => format!("not allowed: {info}"),
+            GpuControllerError::InvalidSysFS => "invalid SysFS".to_owned(),
+            GpuControllerError::ParseError(error) => format!("parse error: {error}"),
+            GpuControllerError::IoError(error) => format!("io error: {error}"),
+        })
+    }
+}
